
'===============================================================================
' Author ............ Jrgen Schlottke, Schnaich-C.-Str. 46, D-W 2200 Elmshorn
' Purpose ............... Demonstration of chess programming under Turbo Pascal
' Date of release .................................................... 18.01.93
' FreeBasic adaptation ................. Roland Chastain, Sarreguemines, France
'===============================================================================

#define whiteColor +1
#define blackColor -1
#define EOL chr(13, 10)
#define none 0
#define maxListIndex 100

const cNothingValue = 000
const cPawnValue    = 002
const cBishopValue  = 006
const cKnightValue  = 007
const cRookValue    = 010
const cQueenValue   = 019
const cKingValue    = 126
const cBorderValue  = 127

dim shared as integer cPawnMove  (1 to 4) = {+01, +02, -09, +11}
dim shared as integer cBishopMove(1 to 4) = {+11, -11, +09, -09}
dim shared as integer cRookMove  (1 to 4) = {-01, +01, -10, +10}
dim shared as integer cKnightMove(1 to 8) = {+12, +21, +19, +08, -12, -21, -19, -08}
dim shared as integer cKingMove  (1 to 8) = {-01, +01, -10, +10, +11, -11, +09, -09}

#ifndef tBoolean
enum tBoolean
  isTrue  = (0 = 0)
  isFalse = (0 > 0)
end enum
#endif

#ifndef tCastling
enum tCastling
  e1g1 ' K
  e1c1 ' Q
  e8g8 ' k
  e8c8 ' q
end enum
#endif

type tMoveRec
  as integer a ' from square
  as integer b ' to square
  as integer v ' value
end type

type tMoveList
  as tMoveRec array(1 to maxListIndex)
  as integer count
end type

type tBoard
  as integer board(-10 to 109)
  as integer castlingAvailability(e1g1 to e8c8)
  as integer castlingLicence(e1g1 to e8c8)
  as integer enPassantTargetSquare(blackColor to whiteColor)
  as integer activeColor, halfmoveCount, materialBalance, oneKingCaptured
  as tMoveList list
  declare sub Initialize(byref as string = "")
  declare sub MovePiece(byval as integer, byval as integer, byval as integer = cQueenValue)
  declare sub AddMoveToList(byval as integer, byval as integer)
  declare sub GenerateMoves()
  declare sub GenerateCastling(byval as tCastling)
  declare sub GenerateAllMoves()
  declare sub CheckCastlingLicence(byval as integer)
  declare function PositionValue(byval as integer) as integer
  declare function Text() as string
  declare function HtmlText as string
  declare function Check() as integer
end type

type tChessEngine
  as integer depth = 3
  as integer maxDepth = 5
  as tBoard playBoard, startBoard
  as tMoveList list
  declare sub Initialize(byref as string = "")
  declare function RecursiveEvaluation(byref as tBoard, byval as integer, byval as integer, byval as integer) as integer
  declare function UserMove(byref as string, byval as integer = cQueenValue) as integer
  declare function InitialState(byref as tBoard, byval as integer) as integer
  declare function BestMove(byval as integer) as integer
  declare function EngineMove() as string
end type

function MoveToStr(byval iFrom as integer, byval iTo as integer) as string
  dim result as string = str(iFrom * 100 + iTo)
  result[0] += 48
  result[2] += 48
  function = result
end function ' MoveToStr

function HtmlBegin(font as string = "chess alfonso-x", size as string = "32px", colour as string = "midnightblue") as string
  function = _
  "<!DOCTYPE html>" & EOL & _
  "<html>" & EOL & _
  "<head>" & EOL & _
  "  <title>JSCP</title>" & EOL & _
  "  <style type=""text/css"">" & EOL & _
  "  p {" & EOL & _
  "    font-family: " & font & ";" & EOL & _
  "    font-size: " & size & ";" & EOL & _
  "    color: " & colour & ";" & EOL & _
  "  }" & EOL & _
  "  </style>" & EOL & _
  "</head>" & EOL & _
  "<body>"
end function

function HtmlEnd as string
  function = _
  "</body>" & EOL & _
  "</html>"
end function

sub tBoard.Initialize(byref s as string = "")
  dim as integer x, y, z
  for z = -10 to 109
    board(z) = cBorderValue
  next z
  if len(s) = 0 then
    restore defaultPlacement
    for y = 8 to 1 step -1
      for x = 1 to 8
        read board(10 * x + y)
      next x
    next y
    castlingAvailability(e1g1) = isTrue
    castlingAvailability(e1c1) = isTrue
    castlingAvailability(e8g8) = isTrue
    castlingAvailability(e8c8) = isTrue
    enPassantTargetSquare(whiteColor) = none
    enPassantTargetSquare(blackColor) = none
    materialBalance = 0
    activeColor = whiteColor
    halfmoveCount = 0
  else
    dim as string symbol = "kqrbnp PNBRQK", s1 = s, a(1 to 6)
    z = instr(s1, " ") : a(1) = left(s1, z - 1) : s1 = right(s1, len(s1) - z)
    z = instr(s1, " ") : a(2) = left(s1, z - 1) : s1 = right(s1, len(s1) - z)
    z = instr(s1, " ") : a(3) = left(s1, z - 1) : s1 = right(s1, len(s1) - z)
    z = instr(s1, " ") : a(4) = left(s1, z - 1) : s1 = right(s1, len(s1) - z)
    z = instr(s1, " ") : a(5) = left(s1, z - 1) : s1 = right(s1, len(s1) - z)
    a(6) = s1
    x = 1 : y = 8
    for z = 1 to len(a(1))
      if a(1)[z - 1] = asc("/") then
        x = 1
        y -= 1
      end if
      if (a(1)[z - 1] >= asc("1")) andalso (a(1)[z - 1] <= asc("8")) then
        do while a(1)[z - 1] > asc("0")
          board(10 * x + y) = cNothingValue
          a(1)[z - 1] -= 1
          x += 1
        loop
      end if
      if instr(symbol, chr(a(1)[z - 1])) > 0 then
        select case instr(symbol, chr(a(1)[z - 1])) - 7
          case -6 : board(10 * x + y) = blackColor * cKingValue
          case -5 : board(10 * x + y) = blackColor * cQueenValue
          case -4 : board(10 * x + y) = blackColor * cRookValue
          case -3 : board(10 * x + y) = blackColor * cBishopValue
          case -2 : board(10 * x + y) = blackColor * cKnightValue
          case -1 : board(10 * x + y) = blackColor * cPawnValue
          case +1 : board(10 * x + y) = whiteColor * cPawnValue
          case +2 : board(10 * x + y) = whiteColor * cKnightValue
          case +3 : board(10 * x + y) = whiteColor * cBishopValue
          case +4 : board(10 * x + y) = whiteColor * cRookValue
          case +5 : board(10 * x + y) = whiteColor * cQueenValue
          case +6 : board(10 * x + y) = whiteColor * cKingValue
        end select
        x += 1
      end if
    next z
    for y = 8 to 1 step -1
      for x = 1 to 8
        materialBalance += board(10 * x + y)
      next x
    next y
    activeColor = iif(a(2) = "w", whiteColor, blackColor)
    castlingAvailability(e1g1) = (instr(a(3), "K") > 0)
    castlingAvailability(e1c1) = (instr(a(3), "Q") > 0)
    castlingAvailability(e8g8) = (instr(a(3), "k") > 0)
    castlingAvailability(e8c8) = (instr(a(3), "q") > 0)
    if len(a(4)) = 2 then
      enPassantTargetSquare(-1 * activeColor) = 10 * (a(4)[0] - 96) + (a(4)[1] - 48)
    else
      enPassantTargetSquare(-1 * activeColor) = none
    end if
    enPassantTargetSquare(activeColor) = none
    halfmoveCount = 2 * (valint(a(6)) - 1) + iif(activeColor = blackColor, 1, 0)
  end if
  oneKingCaptured = isFalse
end sub ' Initialize

sub tBoard.MovePiece(byval iFrom as integer, byval iTo as integer, byval promotion as integer = cQueenValue)
  if abs(board(iFrom)) = cKingValue then
    if sgn(board(iFrom)) = whiteColor then
      castlingAvailability(e1g1) = isFalse
      castlingAvailability(e1c1) = isFalse
    else
      castlingAvailability(e8g8) = isFalse
      castlingAvailability(e8c8) = isFalse
    end if
  end if
  if abs(board(iFrom)) = cRookValue then
    select case iFrom
      case 81 : castlingAvailability(e1g1) = isFalse
      case 11 : castlingAvailability(e1c1) = isFalse
      case 88 : castlingAvailability(e8g8) = isFalse
      case 18 : castlingAvailability(e8c8) = isFalse
    end select
  end if
  if abs(board(iTo)) = cRookValue then
    select case iTo
      case 81 : castlingAvailability(e1g1) = isFalse
      case 11 : castlingAvailability(e1c1) = isFalse
      case 88 : castlingAvailability(e8g8) = isFalse
      case 18 : castlingAvailability(e8c8) = isFalse
    end select
  end if
  if abs(board(iTo)) = cKingValue then oneKingCaptured = isTrue
  if (abs(iTo - iFrom) = 2) and (abs(board(iFrom)) = cPawnValue) then
    enPassantTargetSquare(activeColor) = iFrom + cPawnMove(1) * activeColor
  else
    enPassantTargetSquare(activeColor) = none
  end if
  if (iTo = enPassantTargetSquare(- 1 * activeColor)) and (board(iFrom) = cPawnValue * activeColor) then
     enPassantTargetSquare(activeColor) = none
     MovePiece(iFrom, (iTo \ 10) * 10 + iFrom mod 10)
     activeColor = -1 * activeColor
     halfmoveCount -= 1
     iFrom = (iTo \ 10) * 10 + iFrom mod 10
  end if
  if ((iFrom >= 51) and (iFrom <= 58)) _
  andalso ((iTo = 71) or (iTo = 31) or (iTo = 78) or (iTo = 38)) _
  andalso (abs(board(iFrom)) = cKingValue) then
    board(iTo) = board(iFrom)
    board(iFrom) = cNothingValue
    if iTo \ 10 = 7 then
      iFrom = (iFrom mod 10) + 80
      iTo = iFrom - 20
    else
      iFrom = (iFrom mod 10) + 10
      iTo = iFrom + 30
    end if
    board(iTo) = board(iFrom)
    board(iFrom) = cNothingValue
  else
    materialBalance = materialBalance - board(iTo)
    board(iTo) = board(iFrom)
    board(iFrom) = cNothingValue
    if ((iTo mod 10 = 1) or (iTo mod 10 = 8)) and (abs(board(iTo))) = cPawnValue then
      board(iTo) = promotion * activeColor
      materialBalance = materialBalance + (promotion - cPawnValue) * activeColor
    end if
  end if
  activeColor = -1 * activeColor
  halfmoveCount += 1
end sub ' MovePiece

sub tBoard.AddMoveToList(byval iFrom as integer, byval iTo as integer)
  if oneKingCaptured andalso abs(board(iTo)) = cKingValue then exit sub
  if list.count = maxListIndex then exit sub
  list.count += 1
  list.array(list.count).a = iFrom
  list.array(list.count).b = iTo
end sub ' AddMoveToList

sub tBoard.GenerateMoves()
  dim as integer iFrom, iTo
  dim as integer i, j, k, l
  list.count = 0
  for j = 1 to 8
    for k = 1 to 8
      iFrom = j * 10 + k
      if board(iFrom) <> cNothingValue then
        select case board(iFrom) * activeColor
        case cPawnValue
          iTo = iFrom + cPawnMove(1) * activeColor
          if board(iTo) = cNothingValue then
            AddMoveToList(iFrom, iTo)
            if (activeColor = 1) and (k = 2) or (activeColor = blackColor) and (k = 7) then
              iTo = iFrom + cPawnMove(2) * activeColor
              if (board(iTo) = cNothingValue) then
                AddMoveToList(iFrom, iTo)
              end if
            end if
          end if
          for i = 3 to 4
            iTo = iFrom + cPawnMove(i) * activeColor
            if ((-1 * board(iTo) * activeColor >= cPawnValue) _
            and (-1 * board(iTo) * activeColor <= cKingValue)) _
            or (iTo = enPassantTargetSquare(-1 * activeColor)) then
              AddMoveToList(iFrom, iTo)
            end if
          next i
        case cKnightValue
          for i = 1 to 8
            iTo = iFrom + cKnightMove(i)
            if (-1 * board(iTo) * activeColor >= cNothingValue) _
            and (-1 * board(iTo) * activeColor <= cKingValue) then
              AddMoveToList(iFrom, iTo)
            end if
          next i
        case cBishopValue
          for i = 1 to 4
            iTo = iFrom
            do
              iTo += cBishopMove(i)
              if (-1 * board(iTo) * activeColor >= cNothingValue) _
              and (-1 * board(iTo) * activeColor <= cKingValue) then
                AddMoveToList(iFrom, iTo)
              end if
            loop until board(iTo) <> cNothingValue
          next i
        case cRookValue
          for i = 1 to 4
            iTo = iFrom
            do
              iTo += cRookMove(i)
              if (-1 * board(iTo) * activeColor >= cNothingValue) _
              and (-1 * board(iTo) * activeColor <= cKingValue) then
                AddMoveToList(iFrom, iTo)
              end if
            loop until board(iTo) <> cNothingValue
          next i
        case cKingValue
          for i = 1 to 8
            iTo = iFrom + cKingMove(i)
            if (-1 * board(iTo) * activeColor >= cNothingValue) _
            and (-1 * board(iTo) * activeColor <= cKingValue) then
              l = 0
              do
                l += 1
              loop until (-1 * board(iTo + cKingMove(l)) * activeColor = cKingValue) or (l = 8)
              if l = 8 then AddMoveToList(iFrom, iTo)
            end if
          next i
        case cQueenValue
          for i = 1 to 8
            iTo = iFrom
            do
              iTo += cKingMove(i)
              if (-1 * board(iTo) * activeColor >= cNothingValue) _
              and (-1 * board(iTo) * activeColor <= cKingValue) then
                AddMoveToList(iFrom, iTo)
              end if
            loop until board(iTo) <> cNothingValue
          next i
        end select
      end if
    next k
  next j
end sub ' GenerateMoves

sub tBoard.GenerateCastling(byval aCastling as tCastling)
  dim as integer kingFrom, kingStep, rookFrom, i
  select case as const aCastling
    case e1g1
      kingFrom = 51
      kingStep = +10
      rookFrom = 81
    case e1c1
      kingFrom = 51
      kingStep = -10
      rookFrom = 11
    case e8g8
      kingFrom = 58
      kingStep = +10
      rookFrom = 88
    case e8c8
      kingFrom = 58
      kingStep = -10
      rookFrom = 18
  end select
  i = kingFrom + kingStep
  do
    if board(i) <> cNothingValue then exit sub
    i += kingStep
  loop until i = rookFrom
  AddMoveToList(kingFrom, kingFrom + 2 * kingStep)
end sub

sub tBoard.GenerateAllMoves()
  CheckCastlingLicence(activeColor)
  GenerateMoves()
  if activeColor = whiteColor then
    if castlingAvailability(e1g1) andalso castlingLicence(e1g1) then GenerateCastling(e1g1)
    if castlingAvailability(e1c1) andalso castlingLicence(e1c1) then GenerateCastling(e1c1)
  else
    if castlingAvailability(e8g8) andalso castlingLicence(e8g8) then GenerateCastling(e8g8)
    if castlingAvailability(e8c8) andalso castlingLicence(e8c8) then GenerateCastling(e8c8)
  end if
end sub ' GenerateAllMoves

sub tBoard.CheckCastlingLicence(byval aColor as integer)
  dim c as tCastling
  dim y as integer = iif(aColor = whiteColor, 1, 8)
  for c = e1g1 to e8c8
    castlingLicence(c) = isTrue
  next c
  activeColor = -1 * activeColor
  GenerateMoves()
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if list.array(i).b mod 10 = y then
      select case list.array(i).b \ 10
        case is = 5
          castlingLicence(e1g1) = isFalse
          castlingLicence(e1c1) = isFalse
          castlingLicence(e8g8) = isFalse
          castlingLicence(e8c8) = isFalse
        case is > 5
          castlingLicence(e1g1) = isFalse
          castlingLicence(e8g8) = isFalse
        case is < 5
          castlingLicence(e1c1) = isFalse
          castlingLicence(e8c8) = isFalse
      end select
    end if
  next i
end sub ' CheckCastlingLicence

function tBoard.PositionValue(byval aColor as integer) as integer
  return materialBalance * aColor
end function ' PositionValue

function tBoard.Text() as string
  dim as string s = _
  "+ A B C D E F G H " & EOL & _
  "8                 " & EOL & _
  "7                 " & EOL & _
  "6                 " & EOL & _
  "5                 " & EOL & _
  "4                 " & EOL & _
  "3                 " & EOL & _
  "2                 " & EOL & _
  "1                 " & EOL & _
  "+ A B C D E F G H "
  for y as integer = 8 to 1 step -1
    for x as integer = 1 to 8
      select case abs(board(10 * x + y))
        case cNothingValue
          s[20 * (8 - y) + 2 * x + 20] = asc("-")
        case cPawnValue
          s[20 * (8 - y) + 2 * x + 20] = asc("P")
        case cKnightValue
          s[20 * (8 - y) + 2 * x + 20] = asc("N")
        case cBishopValue
          s[20 * (8 - y) + 2 * x + 20] = asc("B")
        case cRookValue
          s[20 * (8 - y) + 2 * x + 20] = asc("R")
        case cQueenValue
          s[20 * (8 - y) + 2 * x + 20] = asc("Q")
        case cKingValue
          s[20 * (8 - y) + 2 * x + 20] = asc("K")
      end select
      if board(10 * x + y) < cNothingValue then
        s[20 * (8 - y) + 2 * x + 20] += 32
      end if
    next x
  next y
  function = s
end function ' Text

function tBoard.HtmlText as string
  dim as string char, s = _
  "<p>" & EOL & _
  "!""""""""""""""""#<br>" & EOL & _
  "$tMvWlVmT%<br>" & EOL & _
  "$OoOoOoOo%<br>" & EOL & _
  "$ + + + +%<br>" & EOL & _
  "$+ + + + %<br>" & EOL & _
  "$ + + + +%<br>" & EOL & _
  "$+ + + + %<br>" & EOL & _
  "$pPpPpPpP%<br>" & EOL & _
  "$RnBqKbNr%<br>" & EOL & _
  "/(((((((()<br>" & EOL & _
  "</p>"
  for x as integer = 1 to 8
    for y as integer = 1 to 8
      select case board(10 * x + y)
        case cNothingValue             : char = " "
        case whiteColor * cPawnValue   : char = "p"
        case whiteColor * cKnightValue : char = "n"
        case whiteColor * cBishopValue : char = "b"
        case whiteColor * cRookValue   : char = "r"
        case whiteColor * cQueenValue  : char = "q"
        case whiteColor * cKingValue   : char = "k"
        case blackColor * cPawnValue   : char = "o"
        case blackColor * cKnightValue : char = "m"
        case blackColor * cBishopValue : char = "v"
        case blackColor * cRookValue   : char = "t"
        case blackColor * cQueenValue  : char = "w"
        case blackColor * cKingValue   : char = "l" 
      end select
      if (x + y) mod 2 = 0 then
        if board(10 * x + y) = cNothingValue then
          char = "+"
        else
          char = chr(asc(char) - (asc("a") - asc("A")))
        end if
      end if
      s[x + 16 * (8 - y) + 22 - 1] = asc(char)
    next y
  next x
  function = s
end function

function tBoard.Check() as integer
  activeColor = -1 * activeColor
  GenerateMoves()
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if abs(board(list.array(i).b)) = cKingValue then
      return isTrue
    end if
  next i
  return isFalse
end function ' Check

sub tChessEngine.Initialize(byref s as string = "")
  startBoard.Initialize("")
  playBoard.Initialize(s)
end sub ' Initialize

function tChessEngine.RecursiveEvaluation(byref aBoard as tBoard, byval aColor as integer, byval aDepth as integer, byval alfa as integer) as integer
  dim as integer v, i
  dim as tBoard brd
  dim as integer beta
  dim as integer halt
  aBoard.GenerateAllMoves()
  beta = -32000 * aBoard.activeColor * aColor
  i = 0
  halt = isFalse
  while (i < aBoard.list.count) and not halt
    i += 1
    brd = aBoard
    brd.MovePiece(aBoard.list.array(i).a, aBoard.list.array(i).b)
    if ((aDepth >= depth) and (aBoard.board(aBoard.list.array(i).b) = cNothingValue)) or (aDepth = maxDepth) then
      v = brd.PositionValue(aColor)
    else
      v = RecursiveEvaluation(brd, aColor, aDepth + 1, beta)
    end if
    if aBoard.activeColor = aColor then
      if v > beta then beta = v
      if beta > alfa then halt = isTrue
    else
      if v < beta then beta = v
      if beta < alfa then halt = isTrue
    end if
    aBoard.list.array(i).v = v
  wend
  RecursiveEvaluation = beta
end function ' RecursiveEvaluation

function tChessEngine.UserMove(byref arg as string, byval promotion as integer = cQueenValue) as integer
  dim as integer result, move, iFrom, iTo
  dim brd as tBoard
  dim aux as string = arg
  if (len(arg) = 4) andalso _
  (arg[0] >= asc("a")) and (arg[0] <= asc("h")) and _
  (arg[1] >= asc("1")) and (arg[1] <= asc("8")) and _
  (arg[2] >= asc("a")) and (arg[2] <= asc("h")) and _
  (arg[3] >= asc("1")) and (arg[3] <= asc("8")) then
    aux[0] -= (16 + 32)
    aux[2] -= (16 + 32)
  else
    return 0
  end if
  result = isFalse
  move = valint(aux)
  playBoard.GenerateAllMoves()
  for i as integer = 1 to playBoard.list.count
    if move = playBoard.list.array(i).a * 100 + playBoard.list.array(i).b then
      result = isTrue
      exit for
    end if
  next i
  if result = isFalse then return result
  iFrom = move \ 100
  iTo = move mod 100
  brd = playBoard
  brd.MovePiece(iFrom, iTo)
  brd.activeColor = -1 * brd.activeColor
  if brd.Check() then
    result = isFalse
  else
    playBoard.MovePiece(iFrom, iTo, promotion)
    playBoard.halfmoveCount += 1
  end if
  function = result
end function ' UserMove

function tChessEngine.InitialState(byref aBoard as tBoard, byval iSquare as integer) as integer
  if aBoard.board(iSquare) = startBoard.board(iSquare) then
    return isTrue
  else
    return isFalse
  end if
end function

function tChessEngine.BestMove(byval aRecursiveEvaluation as integer) as integer
  dim brd as tBoard
  dim as integer i, j, max, p
  list.count = 0
  for i = 1 to playBoard.list.count
    if playBoard.list.array(i).v = aRecursiveEvaluation then
      list.count += 1
      list.array(list.count).a = playBoard.list.array(i).a
      list.array(list.count).b = playBoard.list.array(i).b
      list.array(list.count).v = 0
    end if
  next i
  max = 0
  function = 1
  for i = 1 to list.count
    brd = playBoard
    with list.array(i)
      .v = 0
      p = abs(brd.board(.a))
      if InitialState(brd, .a) then
        .v += 5
        if p = cPawnValue then .v += 2
      end if
      if (p = cKingValue) then .v -= 10
      if (brd.halfmoveCount < 32) then
        if (p = cPawnValue) or (p = cBishopValue) or (p = cKnightValue) then .v += 20
      end if
      if (.a \ 10 = 1) or (.a \ 10 = 8) or (.a mod 10 = 1) or (.a mod 10 = 8) then .v += 2
      if (.b \ 10 = 1) or (.b \ 10 = 8) or (.b mod 10 = 1) or (.b mod 10 = 8) then .v -= 2
      brd = playBoard
      brd.MovePiece(.a, .b)
      if InitialState(brd, .b) then .v -= 10
      brd.activeColor = -1 * brd.activeColor
      brd.GenerateMoves()
      for j = 1 to brd.list.count
        .v += 1
        if brd.board(brd.list.array(j).b) <> cNothingValue then .v += 1
      next j
      brd.activeColor = -1 * brd.activeColor
      brd.GenerateMoves()
      for j = 1 to brd.list.count
        .v -= 1
        if brd.board(brd.list.array(j).b) <> cNothingValue then .v -= 1
      next j
      if .v >= max then
        max = .v
        function = i
      end if
    end with
  next i
end function ' BestMove

function tChessEngine.EngineMove() as string
  dim as integer i, alreadyInCheck
  dim as string result, flag
  i = BestMove(RecursiveEvaluation(playBoard, playBoard.activeColor, 1, 32000))
  playBoard.halfmoveCount += 1
  alreadyInCheck = playBoard.Check()
  playBoard.MovePiece(list.array(i).a, list.array(i).b)
  result = MoveToStr(list.array(i).a, list.array(i).b)
  playBoard.activeColor = -1 * playBoard.activeColor
  if playBoard.Check() then
    if alreadyInCheck then
      flag = "3"
    else
      flag = "2"
    end if
  else
    playBoard.activeColor = -1 * playBoard.activeColor
    if playBoard.Check() then
      flag = "1"
    else
      flag = "0"
    end if
  end if
  result &= "_" & flag
  function = result
end function ' EngineMove

defaultPlacement:
data -010, -007, -006, -019, -126, -006, -007, -010
data -002, -002, -002, -002, -002, -002, -002, -002
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data +002, +002, +002, +002, +002, +002, +002, +002
data +010, +007, +006, +019, +126, +006, +007, +010
