
' FreeBASIC Compiler 0.90.0

#include ".\libraries\mdlanguage\mdlanguage.bi"
#include ".\libraries\sgui\sgui.bas"
#include ".\libraries\sgui\simplegadget.bas"
#include ".\libraries\sgui\clickmapgadget.bas"
#include ".\libraries\sgui\togglegadget.bas"
#include ".\libraries\sgui\promotionbox.bas"
#include ".\libraries\colors\colors.bi"
#include ".\libraries\iniobject\inifile.bi"
#include ".\chess\chess.bas"
#include ".\engine\engine.bas"
#include ".\files.bas"
#include ".\book.bas"

#ifndef false
#define false 0
#define true (not false)
#endif

#define buttonHeight 32

const programName = "Eschecs FreeBasic"
const optionsFile = "options.ini"
const redrawAll = true

declare sub Info(byref as string = "")
declare sub InfoSleep(byref as string)
declare function IsValidStartingSquare(byref as tChessGame, byref as string) as tBoolean
declare function IsPromotionMove(byref as tChessGame, byref as string) as tBoolean
declare sub LoadOptions(byref as string)
declare function TurnFieldName(byref as string) as string
declare sub HighlightMove(byval as integer, byval as integer, byref as string, byval as integer)

screenres 10 + 320 + 10 + 160 + 10, 10 + 320 + 10, 32, 1, 4
windowtitle programName
const textHeight = 14
dim as integer breit, hoch
screeninfo breit, hoch
width breit\8, hoch\textHeight
#include "colors.bas"
#include "pieces.bas"
InitGFX
Info()

dim language as string
LoadOptions(language)

LoadCustomLanguage(default, language, "language\")
dim sNew as string = GetTranslation(default, "new")
dim sTurn as string = GetTranslation(default, "turn")
dim sQuit as string = GetTranslation(default, "quit")
dim sCheck as string = GetTranslation(default, "check")
dim sWhiteToMove as string = GetTranslation(default, "white")
dim sBlackToMove as string = GetTranslation(default, "black")
dim sComputerPlaysWhite as string = GetTranslation(default, "computer_white")
dim sComputerPlaysBlack as string = GetTranslation(default, "computer_black")
dim sState(gsCheckmate to gsDraw) as string
sState(gsCheckmate) = GetTranslation(default, StateToStr(gsCheckmate))
sState(gsStalemate) = GetTranslation(default, StateToStr(gsStalemate))
sState(gsDraw) = GetTranslation(default, StateToStr(gsDraw))
if language = "de" then
  sWhiteToMove = "Wei" & chr(225) & " am Zug"
  sComputerPlaysWhite = "Wei" & chr(225) & " automatisch"
end if

dim as eventHandle ptr event
dim as gadget ptr button1, button2, button3
dim as gadget ptr boardClickSensor
dim as gadget ptr toggle1, toggle2
dim as Gadget ptr list
#include "field.bas"

dim as integer computerPlaysWhite = false, computerPlaysBlack = false

event = CreateEventHandle
button1 = AddSimpleGadget(event, 340, 10 + 1 * 40, 160, buttonHeight, sNew)
button2 = AddSimpleGadget(event, 340, 10 + 4 * 40, 160, buttonHeight, sTurn)
button3 = AddSimpleGadget(event, 340, 10 + 5 * 40, 160, buttonHeight, sQuit)
boardClickSensor = AddClickMapGadget(event, 10, 10, "bitmap\clickmap.bmp")
toggle1= AddToggleGadget(event, 340, 10 + 2 * 40, 160, buttonHeight, computerPlaysWhite, sComputerPlaysWhite)
toggle2= AddToggleGadget(event, 340, 10 + 3 * 40, 160, buttonHeight, computerPlaysBlack, sComputerPlaysBlack)
GadgetOn(button1, toggle2)

dim pieces as tChessPieces
pieces.Initialize()
color boardColor, boardBkColor
pieces.DrawBoard(10, 10, string(64, 32))

dim game as tChessGame
dim move as string = ""
dim state as tGameState = gsCheckmate
dim check as tBoolean = isFalse
dim bookMoveAvailable as integer

dim as tFileOfString history, logFile
history.Initialize("history.fen")
logFile.Initialize("log.txt")
logFile.AddLine(programName)
logFile.AddLine(date)
logFile.AddLine(time)
logFile.AddLine("")

dim turnBoard as integer = false

#macro UpdateAll()
game.GenerateGeometricalMoves()
game.GenerateSpecialMoves()
game.CheckMovesLegality()
state = game.GameState
if state = gsInProgress then
  with history
    if .Found(.LastLine()) = 3 then state = gsDraw
  end with
end if
pieces.DrawBoard(10, 10, game.String64, turnBoard, redrawAll)
if len(move) = 4 then HighlightMove(10, 10, move, turnBoard)
if state = gsInProgress then
  check = game.Check
  if check then InfoSleep(sCheck)
  Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
else
  Info(sState(state))
end if
#endmacro

dim JSCP as tChessEngine
dim bookLoaded as integer = (LoadBook() = 0)
dim promotion as integer

do
  event->xSleep(1)
  if state = gsInProgress then
    if (computerPlaysWhite and (game.activeColor = cWhite)) _
    or (computerPlaysBlack and (game.activeColor = cBlack)) then
      bookMoveAvailable = false
      if BookLoaded then
        move = BookMove(game.EPD)
        if (len(move) = 4) andalso game.IsLegal(move) then
          bookMoveAvailable = true
          game.DoMove(move)
          JSCP.UserMove(move)
          logFile.AddLine(move & " book")
          history.AddLine(game.FEN)
          UpdateAll()
        else
          rem
        end if
        move = ""
      end if
      if bookMoveAvailable = false then
        move = JSCP.EngineMove
        if game.IsLegal(move) then
          game.DoMove(move)
          move = left(move, 4)
          logFile.AddLine(move & " engine")
          history.AddLine(game.FEN)
          UpdateAll()
        else
          logFile.AddLine("Error: The engine returned an illegal move (" & move & "). Engine stopped.")
          computerPlaysWhite = false
          computerPlaysBlack = false
        end if
        move = ""
      end if
    end if
  end if
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case button1
        game.Initialize()
        history.Initialize()
        history.AddLine(game.FEN)
        UpdateAll()
        JSCP.Initialize
      case button2
        turnBoard = not turnBoard
        pieces.DrawBoard(10, 10, game.String64, turnBoard, redrawAll)
      case button3
        event->EXITEVENT = 1
      case boardClickSensor
        field_ID = getClickMapColor(boardClickSensor)
        if state = gsInProgress then
          if turnBoard = false then
            move += Board(field_ID).fieldname
          else
            move += TurnFieldName(Board(field_ID).fieldname)
          end if
        end if
        if len(move) = 2 then
          if IsValidStartingSquare(game, move) then
            Info()
          else
            beep
            Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
            move = ""
          end if
        end if
        if len(move) = 4 then
          if game.IsLegal(move) then
            if IsPromotionMove(game, move) then
              select case language
                case "de"
                  promotion = PromotionBox(97, 139, "Umwandlung", "S", "L", "T", "D")
                case "en"
                  promotion = PromotionBox(97, 139)
                case "fr"
                  promotion = PromotionBox(97, 139, "Promotion", "C", "F", "T", "D")
              end select
              game.DoMove(move, promotion)
              select case promotion
                case cKnight
                  JSCP.UserMove(move, cKnightValue)
                case cBishop
                  JSCP.UserMove(move, cBishopValue)
                case cRook
                  JSCP.UserMove(move, cRookValue)
                case cQueen
                  JSCP.UserMove(move)
              end select
            else
              game.DoMove(move)
              JSCP.UserMove(move)
            end if
            logFile.AddLine(move & " user")
            history.AddLine(game.FEN)
            UpdateAll()
          else
            beep
            Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
          end if
          move = ""
        end if
      case toggle1
        computerPlaysWhite = GetSelect(toggle1)
      case toggle2
        computerPlaysBlack = GetSelect(toggle2)
    end select
  end if
  if event->KEY<>"" then
    select case event->KEY
      case chr(27)
        event->EXITEVENT = 1
    end select
  end if
loop until event->EXITEVENT

DestroyEventHandle(event)
if bookLoaded then CloseBook()
end

sub Info(byref s as string = "")
  const x = 340
  const y = 10
  FillA(x, y, 160, buttonHeight, infoColor, 1)
  draw string((160 - 8 * len(s)) \ 2 + x, y + (buttonHeight - textHeight) \ 2), left(s, 18), infoTextColor
end sub

sub InfoSleep(byref s as string)
  Info(s)
  sleep 1500
end sub

function IsValidStartingSquare(byref g as tChessGame, byref m as string) as tBoolean
  function = (sgn(g.board(m[0] - 96, m[1] - 48)) = g.activeColor)
end function

function IsPromotionMove(byref g as tChessGame, byref m as string) as tBoolean
  function = (abs(g.board(m[0] - 96, m[1] - 48)) = cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

sub LoadOptions(byref aLanguage as string)
  dim myIni as inifobj.iniobj = inifobj.iniobj(optionsFile)
  dim update as integer = false
  select case myIni.ReadString("interface", "language", "")
  	case "de"
  		aLanguage = "de"
  	case "en"
  		aLanguage = "en"
  	case "fr"
  		aLanguage = "fr"
  	case else
      myIni.WriteString("interface", "language", "en")
  		aLanguage = "en"
  		update = true
  end select
  if update then myIni.UpdateFile()
end sub

function TurnFieldName(byref s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  function = chr(asc1 + 96, asc2 + 48)
end function

sub HighlightMove(byval xo as integer, byval yo as integer, byref m as string, byval aFlipBoard as integer)
  const thickLine = false
  const highlightStartingSquare = false
  dim as integer x1, y1, x2, y2
  x1 = m[0] - 96
  y1 = m[1] - 48
  x2 = m[2] - 96
  y2 = m[3] - 48
  if not aFlipBoard then
    x1 = 40 * (x1 - 1) + xo
    y1 = 40 * (8 - y1) + yo
    x2 = 40 * (x2 - 1) + xo
    y2 = 40 * (8 - y2) + yo
  else
    x1 = 40 * (8 - x1) + xo
    y1 = 40 * (y1 - 1) + yo
    x2 = 40 * (8 - x2) + xo
    y2 = 40 * (y2 - 1) + yo
  end if
  if highlightStartingSquare then line(x1, y1)-(x1 + 39, y1 + 39), highlightColor, b
  line(x2, y2)-(x2 + 39, y2 + 39), highlightColor, b
  if thickLine then
    if highlightStartingSquare then line(x1 + 1, y1 + 1)-(x1 + 38, y1 + 38), highlightColor, b
    line(x2 + 1, y2 + 1)-(x2 + 38, y2 + 38), highlightColor, b
  end if
end sub
